
STATUS_OK = "Ok"

TOKEN_PROGRAM_ID = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA"

class cluster_endpoints:
    MAINNET_BETA = "https://api.mainnet-beta.solana.com"
    MAINNET_BETA_SERUM = "https://solana-api.projectserum.com"
    


import json
import requests

STATUS_OK = 200
DEFAULT_HEADERS = {'Content-Type': 'application/json'}

def _return_result(status, response):
    return {
        "status": status,
        "result": response
    }

def request(endpoint, method, params, request_id=1):
    """ returns { "status":status, "result":result }
        result is json loaded (dict or list) 
        if request successful, status should be equal to 200 """
    # log
    print('[D] 4234 >>> sending request:\n\t endpoint: {}, method: {}, params: {}'.format(
        endpoint, method, params
    ))

    try:
        data = json.dumps(
            {
                "jsonrpc": "2.0",
                "id": request_id,
                "method": method,
                "params": params  
            }
        )
        # POST request and look at the response
        response_raw = requests.post(endpoint, headers=DEFAULT_HEADERS, data=data)
        status = response_raw.status_code
        response = json.loads(response_raw.text)

        # status checks and returns
        if status != STATUS_OK:
            print("[D] 50398 >>> error when requesting. Status code: {}".format(status))
            print("\tResponse: {}\n".response)
            return _return_result(status, response)
        try:
            return _return_result(status, response['result'])
        except Exception as e:
            print("[D] 43903 >>> error when requesting. Exception: {}".format(e))
            return _return_result(e, None)
    except Exception as e:
        print("[D] 32985 >>> error when requesting. Exception: {}".format(e))
        return _return_result(e, None)
        


from .values import cluster_endpoints as endpoints
from .values import TOKEN_PROGRAM_ID
from .values import STATUS_OK
from .rpc import request
from . import rpc


def _safe_return(response_raw, fail_result=[]):
    """ checks the status of response_raw.
        if ok, returns { 'status': STATUS_OK, 'result': response result }
        if fail, returns { 'status': response status, 'result': fail_result } 
    """
    try:
        if response_raw['status'] != rpc.STATUS_OK:
            print("[D] 24398 >>> get_signatures call did not return OK status")
            print("\tReturning empty...")
            return { 'status': response_raw['status'], 'result': [] }
    except Exception as e:
        print("[D] 84721 >>> safety check exception: {}\n\t returning empty".format(e))
        return { 'status': e, 'result': fail_result}
    return { 'status': STATUS_OK, 'result': response_raw['result'] }

class Connection():
    """ deals with Solana RPC Connections """
    def __init__(self, endpoint=endpoints.MAINNET_BETA):
        self.endpoint = endpoint

    def get_signatures( self, pubkey, before_sig=None, limit=None, until_sig=None):
        """ get confirmed signatures for pubkey.
            returns {
                'status': values.STATUS_OK | other error status from RPC
                'result': [list_of_signatures] 
            }
        """
        # define parameters for request
        method = "getConfirmedSignaturesForAddress2"
        ops = dict()
        if before_sig: ops['before'] = before_sig
        if until_sig: ops['until'] = until_sig
        if limit: ops['limit'] = limit
        params = [pubkey, ops]

        # get response
        response_raw = request(self.endpoint, method, params)
        return _safe_return(response_raw, [])

    def get_transaction_data(self, signature):
        """ gets human readable transaction data for signature
            returns {
                'status': values.STATUS_OK | other error status from RPC
                'result': {dict of transaction data} 
            }
         """
        method = "getConfirmedTransaction"
        ops = {"encoding": "jsonParsed"}
        params = [signature, ops]

        # make rpc request
        response_raw = request(self.endpoint, method, params)
        return _safe_return(response_raw, dict())

    def get_token_accounts(self, pubkey):
        """ returns list of token accounts for the pubkey """
        method = "getTokenAccountsByOwner"
        ops1 = {"programId": TOKEN_PROGRAM_ID}
        ops2 = {"encoding": "jsonParsed"}
        params = [pubkey, ops1, ops2]
        # make rpc request
        response_raw = request(self.endpoint, method, params)
        return _safe_return(response_raw, list())






from setuptools import setup, find_namespace_packages

with open("README.md", "r") as fh:
    README_MD = fh.read()

setup(
    name="symmetrypy",
    version="0.1.3",
    description=""" Solana RPC API for symmetry """,
    long_description=README_MD,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=[
        # TODO -- packages required when installing this
    ],
    python_requires=">=3.8, <4",
    keywords="symmetry blockchain solana",
    packages=find_namespace_packages(exclude=["tests", "tests.*", "venv", "venv.*"]),
    classifiers=[
        "Development Status :: 1 - Planning",
    ]
)